@@ -22,10 +22,6 @@
 #include "Zend/zend_interfaces.h" /* for zend_class_serialize_deny */
 #include "ext/standard/php_string.h" /* php_trim */
 
-#ifdef __SSE2__
-#include <emmintrin.h>
-#endif
-
 #include "php_yaf.h"
 #include "yaf_application.h"
 #include "yaf_namespace.h"
@@ -92,11 +88,9 @@ static void yaf_loader_obj_free(zend_object *object) /* {{{ */ {
 	if (loader->glibrary) {
 		zend_string_release(loader->glibrary);
 	}
-	if (YAF_LOADER_NAMESPACES(loader)) {
-		if (GC_DELREF(YAF_LOADER_NAMESPACES(loader)) == 0) {
-			GC_REMOVE_FROM_BUFFER(YAF_LOADER_NAMESPACES(loader));
-			zend_array_destroy(YAF_LOADER_NAMESPACES(loader));
-		}
+	if (GC_DELREF(YAF_LOADER_NAMESPACES(loader)) == 0) {
+		GC_REMOVE_FROM_BUFFER(YAF_LOADER_NAMESPACES(loader));
+		zend_array_destroy(YAF_LOADER_NAMESPACES(loader));
 	}
 	if (loader->properties) {
 		if (GC_DELREF(loader->properties) == 0) {
@@ -167,8 +161,6 @@ static zend_array *yaf_loader_get_namespaces(yaf_loader_object *loader) /* {{{ *
 	HashTable *ht;
 	zend_string *name;
 
-	ZEND_ASSERT(YAF_LOADER_NAMESPACES(loader));
-
 	ALLOC_HASHTABLE(ht);
 	zend_hash_init(ht, zend_hash_num_elements(YAF_LOADER_NAMESPACES(loader)), NULL, ZVAL_PTR_DTOR, 0);
 	HT_ALLOW_COW_VIOLATION(YAF_LOADER_NAMESPACES(loader));
@@ -210,10 +202,8 @@ static HashTable *yaf_loader_get_properties(zval *object) /* {{{ */ {
 	}
 	zend_hash_str_update(ht, "global_library:protected", sizeof("global_library:protected") - 1, &rv);
 	
-	if (YAF_LOADER_NAMESPACES(loader)) {
-		ZVAL_ARR(&rv, yaf_loader_get_namespaces(loader));
-		zend_hash_str_update(ht, "namespace:protected", sizeof("namespace:protected") - 1, &rv);
-	}
+	ZVAL_ARR(&rv, yaf_loader_get_namespaces(loader));
+	zend_hash_str_update(ht, "namespace:protected", sizeof("namespace:protected") - 1, &rv);
 
 	ZVAL_BOOL(&rv, yaf_loader_use_spl_autoload(loader));
 	zend_hash_str_update(ht, "use_spl_autoload:protected", sizeof("use_spl_autoload:protected") - 1, &rv);
@@ -268,7 +258,10 @@ yaf_loader_t *yaf_loader_instance(zend_string *library_path) /* {{{ */ {
 		php_error_docref(NULL, E_WARNING, "Failed to register autoload function");
 	}
 	
-	YAF_LOADER_NAMESPACES(loader) = NULL;
+	ALLOC_HASHTABLE(YAF_LOADER_NAMESPACES(loader));
+	zend_hash_init(YAF_LOADER_NAMESPACES(loader), 8, NULL, ZVAL_PTR_DTOR, 0);
+	HT_ALLOW_COW_VIOLATION(YAF_LOADER_NAMESPACES(loader));
+
 	loader->properties = NULL;
 
 	return &YAF_G(loader);
@@ -282,12 +275,6 @@ int yaf_loader_register_namespace(yaf_loader_object *loader, zend_string *class_
 	char *name = ZSTR_VAL(class_name);
 	uint32_t len = ZSTR_LEN(class_name);
 
-	if (UNEXPECTED(!YAF_LOADER_NAMESPACES(loader))) {
-		ALLOC_HASHTABLE(YAF_LOADER_NAMESPACES(loader));
-		zend_hash_init(YAF_LOADER_NAMESPACES(loader), 8, NULL, ZVAL_PTR_DTOR, 0);
-		HT_ALLOW_COW_VIOLATION(YAF_LOADER_NAMESPACES(loader));
-	}
-
 	ZVAL_NULL(&rv);
 	target = YAF_LOADER_NAMESPACES(loader);
 
@@ -350,43 +337,15 @@ int yaf_loader_register_namespace_multi(yaf_loader_object *loader, zval *namespa
 }
 /* }}} */
 
-static void zend_always_inline yaf_loader_replace_chr(char *name, uint32_t len, zend_uchar f, zend_uchar t) /* {{{ */ {
-	char *pos = name;
-#ifdef __SSE2__
-	do {
-		const __m128i from = _mm_set1_epi8(f);
-		const __m128i delta = _mm_set1_epi8(t - f);
-		while (len >= 16) {
-			__m128i op = _mm_loadu_si128((__m128i *)pos);
-			__m128i eq = _mm_cmpeq_epi8(op, from);
-			if (_mm_movemask_epi8(eq)) {
-				eq = _mm_and_si128(eq, delta);
-				op = _mm_add_epi8(op, eq);
-				_mm_storeu_si128((__m128i*)pos, op);
-			}
-			len -= 16;
-			pos += 16;
-		}
-	} while (0);
-#endif
-	if (len) {
-		name = pos; /* reset start */
-		while ((pos = memchr(pos, f, len - (pos - name)))) {
-			*pos++ = t;
-		}
-	}
-}
-/* }}} */
-
 static void yaf_loader_sanitize_path(char *name, uint32_t len) /* {{{ */ {
-	yaf_loader_replace_chr(name, len, '_', DEFAULT_SLASH);
+	yaf_replace_chr(name, len, '_', DEFAULT_SLASH);
 }
 /* }}} */
 
 static void yaf_loader_sanitize_name(char *name, uint32_t len, char *buf) /* {{{ */ {
 	memcpy(buf, name, len);
 	/* replace all '\' to '_' */
-	yaf_loader_replace_chr(buf, len, '\\', '_');
+	yaf_replace_chr(buf, len, '\\', '_');
 }
 /* }}} */
 
@@ -582,11 +541,12 @@ ZEND_HOT static int yaf_loader_load_user(yaf_loader_object *loader, char *buf, u
 	uint32_t origin_len = len;
 	yaf_application_object *app = yaf_application_instance();
 
-	if (!YAF_LOADER_NAMESPACES(loader) ||
-		((library_dir = yaf_loader_resolve_namespace(loader, buf, &len)) == ((zend_string*)-1))) {
-		library_dir = loader->library;
-	} else if (library_dir) {
-		name += (origin_len - len);
+	if ((library_dir = yaf_loader_resolve_namespace(loader, buf, &len))) {
+		if (library_dir == ((zend_string*)-1)) {
+			library_dir = loader->library;
+		} else {
+			name += (origin_len - len);
+		}
 	} else {
 		if (!loader->glibrary) {
 			library_dir = loader->library;
@@ -835,11 +795,7 @@ PHP_METHOD(yaf_loader, getLocalNamespace) {
 		return;
 	}
 
-	if (YAF_LOADER_NAMESPACES(loader)) {
-		ZVAL_ARR(return_value, yaf_loader_get_namespaces(loader));
-	} else {
-		RETURN_NULL();
-	}
+	ZVAL_ARR(return_value, yaf_loader_get_namespaces(loader));
 }
 /* }}} */
 
@@ -852,9 +808,7 @@ PHP_METHOD(yaf_loader, clearLocalNamespace) {
 		return;
 	}
 
-	if (YAF_LOADER_NAMESPACES(loader)) {
-		zend_hash_clean(YAF_LOADER_NAMESPACES(loader));
-	}
+	zend_hash_clean(YAF_LOADER_NAMESPACES(loader));
 
 	RETURN_TRUE;
 }
@@ -913,11 +867,12 @@ PHP_METHOD(yaf_loader, getNamespacePath) {
 		sanitized_name = do_alloca(sanitized_len, use_heap);
 		yaf_loader_sanitize_name(ZSTR_VAL(name), sanitized_len, sanitized_name);
 	}
-	if (!YAF_LOADER_NAMESPACES(loader) ||
-		((path = yaf_loader_resolve_namespace(loader, sanitized_name, &sanitized_len)) == (zend_string*)-1)) {
-		RETVAL_STR_COPY(loader->library);
-	} else if (path) {
-		RETVAL_STR_COPY(path);
+	if ((path = yaf_loader_resolve_namespace(loader, sanitized_name, &sanitized_len))) {
+		if (path == ((zend_string*)-1)) {
+			RETVAL_STR_COPY(loader->library);
+		} else {
+			RETVAL_STR_COPY(path);
+		}
 	} else {
 		if (loader->glibrary) {
 			RETVAL_STR_COPY(loader->glibrary);
